
/* 
 * Copyright (c) 2003, Tetsuhiko Koto.
 * Licensed under the GPL.
 * 
 */

import java.util.*;
import java.util.regex.*;
import java.io.*;
import org.apache.tools.ant.*;
import org.apache.tools.ant.types.*;
import org.apache.tools.ant.taskdefs.*;
import org.apache.tools.ant.util.*;

public class HTMLJavaDocTask extends Javadoc {
    private Path depsrc = null;
    public void setDepsrc(Path path) {
        depsrc = path;
    }
    
    private File depdest = null;
    public void setDestdir(File dir) {
        super.setDestdir(dir);
        depdest = dir;
    }
    
    private String encoding;
    public void setDocencoding(String encoding) {
        super.setDocencoding(encoding);
        this.encoding = encoding;
    }
    
    private List replacements = null;
    public Replacement createReplacement() {
        if (replacements == null)
            replacements = new LinkedList();
        Replacement result = new Replacement();
        replacements.add(result);
        return result;
    }
    public static class Replacement {
        private Pattern old;
        public void setOld(String old) {
            this.old = Pattern.compile(old);
        }
        private String hot;
        public void setNew(String hot) {
            this.hot = hot;
        }
    }
    
    public void execute() throws BuildException {
        String[] list = depsrc.list();
        for (int j = 0; j < list.length; j++) {
            File base = project.resolveFile(list[j]);
            
            DirectoryScanner scanner = new DirectoryScanner();
            scanner.setBasedir(base);
            scanner.setIncludes(new String[] { "**" });
            scanner.scan();
            String[] files = scanner.getIncludedFiles();
            
            MergingMapper mapper = new MergingMapper();
            mapper.setTo(new File(depdest, "index.html").toString());
            
            files = new SourceFileScanner(this).restrict(files, base, depdest, mapper);
            if (files.length > 0) {
                super.execute();
                try {
                    if (replacements != null)
                        postprocess(depdest);
                } catch (IOException x) {
                    throw new BuildException(x);
                }
                return;
            }
        }
    }
    
    private void postprocess(File file) throws IOException {
        if (file.isDirectory()) {
            File[] files = file.listFiles();
            for (int i = 0; i < files.length; i++)
                postprocess(files[i]);
            return;
        }
        
        if (file.getName().endsWith(".html"))
            replace(file);
    }
    
    private void replace(File file) throws IOException {
        StringBuffer master = new StringBuffer((int) Math.max(10, Math.min(Integer.MAX_VALUE, file.length())));
        Reader reader = new InputStreamReader(new FileInputStream(file), encoding);
        char[] buffer = new char[1024];
        for(;;) {
            int size = reader.read(buffer);
            if (size < 0)
                break;
            master.append(buffer, 0, size);
        }
        reader.close();
        
        Replacement[] replacements = (Replacement[]) this.replacements.toArray(new Replacement[0]);
        Matcher[] matchers = new Matcher[replacements.length];
        int[] indexes = new int[replacements.length];
        for (int i = 0; i < replacements.length; i++) {
            matchers[i] = replacements[i].old.matcher(master);
            indexes[i] = -1;
        }
        
        StringBuffer result = new StringBuffer(master.length() * 2);
        int index = 0;
    loop:
        for (;;) {
            int min = master.length();
            for (int j = 0; j < indexes.length; j++) {
                if (indexes[j] < index) {
                    if (matchers[j].find(index)) {
                        indexes[j] = matchers[j].start();
                        //System.out.println("**match** " + matchers[j].pattern().pattern() + "  //  " + master.subSequence(matchers[j].start(), matchers[j].end()));
                    } else {
                        indexes[j] = master.length();
                    }
                }
                min = Math.min(min, indexes[j]);
            }
            
            //System.out.println("**skip** " + master.subSequence(index, min));
            result.append(master.subSequence(index, min));
            index = min;
            if (index == master.length())
                break;
            
            for (int j = 0; j < indexes.length; j++) {
                if (indexes[j] == index) {
                    //System.out.println("**old** " + master.subSequence(matchers[j].start(), matchers[j].end()));
                    StringBuffer replacement = new StringBuffer();
                    Replacement rep = replacements[j];
                    for (int k = 0; k < rep.hot.length(); k++) {
                        char ch = rep.hot.charAt(k);
                        if (ch != '\\' || k == rep.hot.length() - 1) {
                            replacement.append(ch);
                        } else {
                            char next = rep.hot.charAt(++k);
                            if (next == ch) {
                                replacement.append(ch);
                            } else {
                                int group = Character.digit(next, Character.MAX_RADIX);
                                if (group == -1) {
                                    replacement.append(ch);
                                    k--;
                                } else {
                                    String str = matchers[j].group(group);
                                    if (str != null)
                                        replacement.append(str);
                                }
                            }
                        }
                    }
                    result.append(replacement);
                    //System.out.println("**new** " + replacement);
                    index = matchers[j].end();
                    break;
                }
            }
        }
        
        Writer writer = new OutputStreamWriter(new FileOutputStream(file), encoding);
        writer.write(result.toString());
        writer.close();
    }
    
    private static boolean startsWith(StringBuffer buffer, int index, String str) {
        if (index + str.length() > buffer.length())
            return false;
        
        return buffer.substring(index, index + str.length()).equals(str);
    }
    
}

