
public class JavaParser extends JavaSyntax {
    private final boolean supportAssertion;
    public JavaParser(boolean supportAssertion) {
        this.supportAssertion = supportAssertion;
    }
    
    protected LexicalAnalyzer createLexicalAnalyzer(String sourceName, CharSequence text, int tabStop) throws ParseException {
        return new Default.LexicalAnalyzer(sourceName, text, tabStop) {
            protected char nextChar() throws ParseException {
                return super.nextUnicodeEscapedChar();
            }
            public Token next() throws ParseException {
                Token result = super.next();
                if (!supportAssertion && result.getSymbolID() == TOKEN_ASSERT) {
                    result = new Default.Token(TOKEN_IDENTIFIER, result.getImage(), result.getOriginalImage(), result);
                }
                return result;
            }
        };
    }
    
    protected Node createNode(int symbolID, NodeInitializationParameters parameters) throws ParseException {
        return createNode(symbolID, parameters, true);
    }
    
    public static void main(String[] args) {
        try {
            CompilationUnit root = new JavaParser(true).parseCompilationUnit(new java.io.File(args[0]), null);
            System.out.println(root);
        } catch(Exception x) {
            x.printStackTrace();
        }
    }
}

